# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#     https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.
# pylint:disable=missing-module-docstring

import factory

from qctrlcommons.serializers import encode
from qctrlcommons.utils import get_nested_value


def segment_type(self):
    """A function to get the segment type base on its attributes
    """

    def get_segment_type(control_type):
        """determine the correct segment type based on the control type.
        args:
            control_type (str): a control type that defines what a pulse type is.

        returns:
            str: the pulse type or none if an unknown type is found.
        """
        if control_type == "drive":
            return "complex"
        if control_type == "shift":
            return "real"

        return None

    # assign correct type if a control exists.
    if hasattr(self, "control"):
        return get_segment_type(self.control.fields["type"])
    if hasattr(self.factory_parent, "type"):
        return get_segment_type(self.factory_parent.type)

    return None


def get_config(ref, default=None):
    """Returns nested config values.
    """
    return get_nested_value(CONFIG, ref, default)


CONFIG = {
    "IGNORE_FACTORIES": (
        "actions",
        "cards",
        "plans",
        "products",
        "subscriptions",
        "users",
    ),
    "FACTORIES": {
        "PulseFactory": {
            "attrs": {"segment_type": factory.lazy_attribute(segment_type)},
            "doc": "If segment_type is not provided, it will be automatically "
            "set according to the related control.",
        },
        "GraphFactory": {
            "preprocess": {
                "definition": encode,
                # NOTE: this is to avoid a bug with factoryboy. In builder.py:163,
                # the v is a pythonflow obj which has overridden eq and getattr
                # functionality. This results in that line always evaluating to
                # true-like and the field being added as a post-declaration which
                # fails validation if the field is required.
            }
        },
    },
}
