# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#     https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.
# pylint:disable=missing-module-docstring
import logging
import re

from qctrlcommons.utils import get_nested_value


LOGGER = logging.getLogger(__name__)


def separate_list(source: list, func) -> tuple:
    """
    Returns two lists. First list will only
    include items which have a non-zero result
    when calling fn; second list will be every
    other item in the source.
    """
    matched = []
    unmatched = []

    for item in source:
        if func(item):
            matched.append(item)
        else:
            unmatched.append(item)

    return matched, unmatched


def get_resource_schema(schema: dict, resource_name: str) -> dict:
    """Given the API schema, returns the schema
    for the given resource (if exists).
    """
    return get_nested_value(schema, "components.schemas.{}".format(resource_name), {})


def normalize_encoded_field_name(
    name: str = None, many: list = None, dict_keys: dict = None
):
    """A function to normalize the field name based on the
    passing data type (str, list or dict)
    """
    pattern = re.compile("_encoded$")

    if name:
        return pattern.sub("", name)

    if many:
        return [normalize_encoded_field_name(x) for x in many]

    if dict_keys:
        result_dict_keys = {}
        for key in dict_keys.keys():
            new_key = normalize_encoded_field_name(key)
            result_dict_keys.update({new_key: dict_keys[key]})

        return result_dict_keys

    return None


def _is_private_field(name: str) -> bool:
    """Checks if the GraphQL field is private
    and should not be accessed by the user.
    """
    return name == "_core_type"
